@@ -18,12 +18,16 @@
 
 #include "gdf_mex.h"
 #include "matlab_tools/mxStructAccess.h"
+#include <GDF/EventConverter.h>
 #include <GDF/Reader.h>
 #include <mex.h>
 
 #include <boost/lexical_cast.hpp>
 
 
+#define VERBOSE
+
+
 using namespace std;
 
 // Calling options:
@@ -43,6 +47,8 @@ using namespace std;
 #define OPTION_DATAFORMAT_COL1  "COL"
 #define OPTION_DATAFORMAT_COL2  "COLUMN"
 
+#define OPTION_CONVERTEVENTS    "FORCEMODE3EVENTS"
+
 enum eMultirateMode
 {
     MR_UPSAMPLE,
@@ -56,6 +62,16 @@ enum eDataOrientation
     DO_COL
 };
 
+enum eVerboseLevels
+{
+    V_NONE = 0,
+    V_CONSTRUCTOR_CALLS,
+    V_FUNCTION_CALLS,
+    V_FUNCTION_DETAILS,
+    V_FUNCTION_LOOPS,
+    V_ALL
+};
+
 // ===========================================================================
 // ===========================================================================
 
@@ -103,6 +119,8 @@ public:
 
     void parseInputArguments( );
 
+    void verboseMessage( int level, std::string message );
+
 private:
 
     size_t nlhs_, nrhs_;
@@ -113,6 +131,7 @@ private:
     eMultirateMode multirate_mode;
     Interpolator *interpolator;
     eDataOrientation data_orientation;
+    bool convert_events;
 
     gdf::uint16 num_signals;
     gdf::uint64 num_records;
@@ -121,6 +140,8 @@ private:
 
     map< gdf::uint32, vector<gdf::uint16> > signals_by_samplerate;
     vector<gdf::uint32> samples_per_record;
+
+    int verbose_level;
 };
 
 // ===========================================================================
@@ -136,9 +157,12 @@ void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
 // ===========================================================================
 
 CmexObject::CmexObject( size_t nlhs, mxArray *plhs[], size_t nrhs, const mxArray *prhs[] )
+    : verbose_level( V_NONE )
 {
     using boost::numeric_cast;
 
+    verboseMessage( V_CONSTRUCTOR_CALLS, "entering CmexObject::CmexObject( );");
+
     interpolator = NULL;
 
     if (nrhs == 0)
@@ -157,6 +181,7 @@ CmexObject::CmexObject( size_t nlhs, mxArray *plhs[], size_t nrhs, const mxArray
     multirate_mode = MR_SINGLE;
     interpolator = new InterpolatorDummy( );
     data_orientation = DO_COL;
+    convert_events = false;
 
     nlhs_ = nlhs;
     plhs_ = plhs;
@@ -164,19 +189,23 @@ CmexObject::CmexObject( size_t nlhs, mxArray *plhs[], size_t nrhs, const mxArray
     prhs_ = prhs;
 
     parseInputArguments( );
+    verboseMessage( V_CONSTRUCTOR_CALLS, "leaving CmexObject::CmexObject( );");
 }
 
 // ===========================================================================
 
 CmexObject::~CmexObject( )
 {
+    verboseMessage( V_CONSTRUCTOR_CALLS, "entering CmexObject::~CmexObject( );");
     if( interpolator ) delete interpolator;
+    verboseMessage( V_CONSTRUCTOR_CALLS, "leaving CmexObject::~CmexObject( );");
 }
 
 // ===========================================================================
 
 void CmexObject::execute( )
 {
+    verboseMessage( V_FUNCTION_CALLS, "entering CmexObject::execute( );");
     gdf::Reader reader;
 
     reader.enableCache( false );
@@ -185,6 +214,8 @@ void CmexObject::execute( )
     num_signals = reader.getMainHeader_readonly().get_num_signals();
     num_records = reader.getMainHeader_readonly().get_num_datarecords();
 
+    verboseMessage( V_FUNCTION_DETAILS, "Number of data records: "+boost::lexical_cast<std::string>( num_records ) );
+
     if( num_signals > 0 )
     {
 
@@ -237,12 +268,15 @@ void CmexObject::execute( )
     }
 
     reader.close( );
+
+    verboseMessage( V_FUNCTION_CALLS, "leaving CmexObject::execute( );");
 }
 
 // ===========================================================================
 
 void CmexObject::getUpsampleData( gdf::Reader &reader )
 {
+    verboseMessage( V_FUNCTION_CALLS, "entering CmexObject::getUpsampleData( );");
     // construct output structure
     if( data_orientation == DO_ROW )
         plhs_[0] = mxCreateNumericMatrix( num_signals, num_records*max_rate, mxDOUBLE_CLASS, mxREAL );
@@ -288,13 +322,17 @@ void CmexObject::getUpsampleData( gdf::Reader &reader )
                 interpolator->expand( &data[s*num_records*max_rate], 1, samples_per_record[s]*num_records, max_rate*num_records );
         }
     }
+    verboseMessage( V_FUNCTION_CALLS, "leaving CmexObject::getUpsampleData( );");
 }
 
 // ===========================================================================
 
 void CmexObject::getGroupData( gdf::Reader &reader )
 {
+    verboseMessage( V_FUNCTION_CALLS, "entering CmexObject::getGroupData( );");
+
     // construct output structure
+    verboseMessage( V_FUNCTION_DETAILS, "   constructing output structure.");
 
     plhs_[0] = mxCreateStructMatrix( num_samplerates, 1, 0, NULL );
     mxAddField( plhs_[0], "channels" );
@@ -303,6 +341,7 @@ void CmexObject::getGroupData( gdf::Reader &reader )
     map< gdf::uint32, vector<gdf::uint16> >::iterator it = signals_by_samplerate.begin( );
     for( gdf::uint16 g=0; g<num_samplerates; g++, it++ )
     {
+        verboseMessage( V_FUNCTION_LOOPS, "     creating group." );
         gdf::uint16 signal = it->second.front( );   // just one of the signals in this group
         size_t num_samples = samples_per_record[signal] * num_records;
 
@@ -329,8 +368,16 @@ void CmexObject::getGroupData( gdf::Reader &reader )
     }
 
     // fill output structure with data
+    verboseMessage( V_FUNCTION_DETAILS, "   filling output structure." );
+
     for( gdf::uint64 r=0; r<num_records; r++ )
+    //for( gdf::uint64 r=0; r<100; r++ )
     {
+        if( r==0 )
+            verboseMessage( V_FUNCTION_LOOPS, "     1st record" );
+        else if( r==num_records-1 )
+            verboseMessage( V_FUNCTION_LOOPS, "     last record" );
+
         gdf::Record *rec = reader.getRecordPtr( r );
         it = signals_by_samplerate.begin( );
         for( gdf::uint16 g=0; g<num_samplerates; g++, it++ )
@@ -338,8 +385,8 @@ void CmexObject::getGroupData( gdf::Reader &reader )
             if( it->first == 0 )
                 continue;
             double *data = mxGetPr( mx::getField( plhs_[0], "data", g+1 ) );
-            size_t rows = it->second.size( );
-            for( size_t s=0; s<rows; s++ )
+            size_t sing = it->second.size( );	// signals in group
+            for( size_t s=0; s<sing; s++ )
             {
                 gdf::uint16 signal = it->second[s];
                 for( gdf::uint32 n=0; n<samples_per_record[signal]; n++ )
@@ -347,23 +394,29 @@ void CmexObject::getGroupData( gdf::Reader &reader )
                     if( data_orientation == DO_ROW )
                     {
                         size_t column = n + r * samples_per_record[signal];
-                        data[s+column*rows] = rec->getChannel(signal)->getSamplePhys( n );
+			size_t row = s;
+			size_t rows = sing;
+                        data[row+column*rows] = rec->getChannel(signal)->getSamplePhys( n );
                     }
                     else
                     {
-                        size_t row = n + r * samples_per_record[s];
-                        data[row+s*num_records*samples_per_record[signal]] = rec->getChannel( s )->getSamplePhys( n );
+			size_t column = s;
+                        size_t row = n + r * samples_per_record[signal];
+			size_t rows = num_records * samples_per_record[signal];
+                        data[row+column*rows] = rec->getChannel(signal)->getSamplePhys( n );
                     }
                 }
             }
         }
     }
+    verboseMessage( V_FUNCTION_CALLS, "leaving CmexObject::getGroupData( );");
 }
 
 // ===========================================================================
 
 void CmexObject::getSingleData( gdf::Reader &reader )
 {
+    verboseMessage( V_FUNCTION_CALLS, "entering CmexObject::getSingleData( );");
     // construct output structure
     plhs_[0] = mxCreateCellMatrix( num_signals, 1 );
     for( gdf::uint16 s=0; s<num_signals; s++ )
@@ -393,17 +446,30 @@ void CmexObject::getSingleData( gdf::Reader &reader )
             }
         }
     }
+    verboseMessage( V_FUNCTION_CALLS, "leaving CmexObject::getSingleData( );");
 }
 
 // ===========================================================================
 
 void CmexObject::loadEvents( gdf::Reader &reader )
 {
+    verboseMessage( V_FUNCTION_CALLS, "entering CmexObject::loadEvents( );");
     plhs_[2] = mxCreateStructMatrix( 1, 1, 0, NULL );
     gdf::EventHeader *evh = reader.getEventHeader( );
     gdf::uint32 num_ev = evh->getNumEvents( );
 
-    switch( evh->getMode() )
+    std::vector<gdf::Mode3Event> ev3;
+
+    gdf::uint8 mode = evh->getMode( );
+
+    if( convert_events && mode == 1 )
+    {
+        ev3 = gdf::convertMode1EventsIntoMode3Events( evh->getMode1Events() );
+        mode = 3;
+        num_ev = ev3.size( );
+    }
+
+    switch( mode )
     {
     default: throw invalid_argument( " Invalid Event Mode." );
     case 1: {
@@ -413,7 +479,7 @@ void CmexObject::loadEvents( gdf::Reader &reader )
             mx::setField( plhs_[2], NULL, GDFE_TYP );
 
             mx::setFieldNumeric( plhs_[2], evh->getSamplingRate( ), GDFE_FS );
-            mx::setFieldNumeric( plhs_[2], evh->getMode( ), GDFE_MODE );
+            mx::setFieldNumeric( plhs_[2], mode, GDFE_MODE );
 
             mxArray *mxpos = mxCreateNumericMatrix( 1, num_ev, mxUINT32_CLASS, mxREAL );
             mx::setField( plhs_[2], mxpos, GDFE_POS );
@@ -432,6 +498,10 @@ void CmexObject::loadEvents( gdf::Reader &reader )
             }
         } break;
     case 3: {
+
+            if( !convert_events )
+                ev3 = evh->getMode3Events( );
+
             mx::setField( plhs_[2], NULL, GDFE_MODE );
             mx::setField( plhs_[2], NULL, GDFE_FS );
             mx::setField( plhs_[2], NULL, GDFE_POS );
@@ -440,7 +510,7 @@ void CmexObject::loadEvents( gdf::Reader &reader )
             mx::setField( plhs_[2], NULL, GDFE_3_DUR );
 
             mx::setFieldNumeric( plhs_[2], evh->getSamplingRate( ), GDFE_FS );
-            mx::setFieldNumeric( plhs_[2], evh->getMode( ), GDFE_MODE );
+            mx::setFieldNumeric( plhs_[2], mode, GDFE_MODE );
 
             mxArray *mxpos = mxCreateNumericMatrix( 1, num_ev, mxUINT32_CLASS, mxREAL );
             mx::setField( plhs_[2], mxpos, GDFE_POS );
@@ -461,7 +531,8 @@ void CmexObject::loadEvents( gdf::Reader &reader )
             for( gdf::uint32 e=0; e<num_ev; e++ )
             {
                 gdf::Mode3Event event;
-                evh->getEvent( e, event );
+                //evh->getEvent( e, event );
+                event = ev3[e];
                 positions[e] = event.position;
                 types[e] = event.type;
                 channels[e] = event.channel;
@@ -469,21 +540,25 @@ void CmexObject::loadEvents( gdf::Reader &reader )
             }
         } break;
     }
+    verboseMessage( V_FUNCTION_CALLS, "leaving CmexObject::loadEvents( );");
 }
 
 // ===========================================================================
 
 void CmexObject::constructHeader( gdf::Reader &reader )
 {
+    verboseMessage( V_FUNCTION_CALLS, "entering CmexObject::constructHeader( );");
     plhs_[1] = constructHeaderStruct( num_signals );
 
     Header2Struct( plhs_[1], &reader.getHeaderAccess_readonly() );
+    verboseMessage( V_FUNCTION_CALLS, "leaving CmexObject::constructHeader( );");
 }
 
 // ===========================================================================
 
 void CmexObject::parseInputArguments( )
 {
+    verboseMessage( V_FUNCTION_CALLS, "entering CmexObject::parseInputArguments( );");
     using boost::lexical_cast;
 
     filename = mx::getString( prhs_[0] );
@@ -542,11 +617,26 @@ void CmexObject::parseInputArguments( )
                 else
                     throw invalid_argument( " Unknown Data Orientation: '"+arg+"'" );
             }
+            else if( opt == OPTION_CONVERTEVENTS )
+            {
+                convert_events = true;
+            }
         } catch( mx::Exception &e )
         {
             throw invalid_argument( " While parsing argument "+lexical_cast<string>(n+1)+": "+e.what() );
         }
     }
+    verboseMessage( V_FUNCTION_CALLS, "leaving CmexObject::parseInputArguments( );");
+}
+
+// ===========================================================================
+
+void CmexObject::verboseMessage( int level, std::string message )
+{
+#ifdef VERBOSE
+    if( level <= verbose_level )
+        mexPrintf( ( "%d : " + message + "\n").c_str(), level );
+#endif //VERBOSE
 }
 
 // ===========================================================================
